<?php

/**
 * @file
 * eurobank redirect menu items.
 */

/**
 * Hook function for validation.
 */
function commerce_eurobank_redirect_validate() {
  // Get the _POST data from eurobank.
  $data = $_POST;

  $currency = trim(check_plain($data['Currency']));
  $merchant_id = check_plain($data['Shop']);
  $amount = number_format(floatval(check_plain($data['Amount'])), 2, '', '');
  $trans_id = check_plain($data['Ref']);
  $t = explode('_', $trans_id);
  $order_id = array_shift($t);

  $validation_success = FALSE;
  $validation_message = 'Never found ';

  $result = db_query("SELECT * FROM {commerce_eurobank_redirect} WHERE order_id = :order_id", array(':order_id' => $order_id));
  foreach ($result as $record) {
    // If at least one found, we are in a good way.
    $validation_success = TRUE;
    $validation_message = '';

    if ($currency != $record->currency) {
      $validation_success = FALSE;
      $validation_message .= ' Currency error:' . $currency . '|' . $record->currency;
    }

    if ($amount != $record->amount) {
      $validation_success = FALSE;
      $validation_message .= ' Amount error:' . $amount . '|' . $record->amount;
    }

    $payment_method = commerce_payment_method_instance_load($record->Method);

    if ($merchant_id != $payment_method['settings']['merchant_id']) {
      $validation_success = FALSE;
      $validation_message .= ' Merchant error:' . $merchant_id . '|' . $payment_method['settings']['merchant_id'];
    }
  }

  $order = commerce_order_load($order_id);
  if (empty($order)) {
    $validation_success = FALSE;
    $validation_message .= ' Order not found';
  }

  if ($validation_success) {
    watchdog('commerce_eurobank_redirect',
      'Order validated [Order: @order]',
      array(
        '@order' => $order_id,
      ),
      WATCHDOG_NOTICE);

    print '[OK]';
    return;
  }
  else {
    watchdog('commerce_eurobank_redirect',
      'Validation error [Order: @order][Error: @error]',
      array(
        '@order' => $order_id,
        '@error' => $validation_message,
      ),
      WATCHDOG_WARNING);

    $transaction = commerce_payment_transaction_new('commerce_eurobank_redirect', $order_id);
    $transaction->instance_id = $trans_id;
    $transaction->payload[REQUEST_TIME] = $data;
    $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
    $transaction->remote_id = -1;
    $transaction->remote_status = 'Validation error';
    $transaction->message = 'Validation error ' . $validation_message;
    $result = commerce_payment_transaction_save($transaction);

    print '[NOT OK]';
    return;
  }
}

/**
 * Hook function for confirmation.
 */
function commerce_eurobank_redirect_confirm() {
  // Get the _POST data from eurobank.
  $data = $_POST;

  $currency = trim(check_plain($data['Currency']));
  $merchant_id = check_plain($data['Shop']);
  $password = trim(check_plain($data['Password']));
  $amount = number_format(floatval(check_plain($data['Amount'])), 2, '', '');
  $trans_id = check_plain($data['Ref']);
  $t = explode('_', $trans_id);
  $order_id = array_shift($t);
  $eurobank_trans_id = check_plain($data['Transid']);

  $confirmation_success = FALSE;
  $confirmation_message = 'Never found ';

  $result = db_query("SELECT * FROM {commerce_eurobank_redirect} WHERE order_id = :order_id", array(':order_id' => $order_id));
  foreach ($result as $record) {
    // If at least one found, we are in a good way.
    $confirmation_success = TRUE;
    $confirmation_message = '';

    if ($currency != $record->currency) {
      $confirmation_success = FALSE;
      $confirmation_message .= ' Currency error:' . $currency . '|' . $record->currency;
    }

    /*
    // Kanonika mporoume na paraleipsoume ton elegxo timis,
    //  giati mporei na exei xrewthei ligotero poso anti tou epithimitou
    //  ..paizei omws ontws na mporei na ginei?
    if ($amount != $record->amount) {
      $confirmation_success = FALSE;
    }
    */
    $payment_method = commerce_payment_method_instance_load($record->Method);
    if ($password != $payment_method['settings']['confirmation_password']) {
      $confirmation_success = FALSE;
      $confirmation_message .= ' Password error:' . $password . '|' . $payment_method['settings']['confirmation_password'];
    }

    /*
    // Den einai to mercant_id alla perigrafiko onoma magaziou
    if ($merchant_id != $payment_method['settings']['merchant_id']) {
      $confirmation_success = FALSE;
      $confirmation_message .= ' Merchant error:' . $merchant_id . '|' . $payment_method['settings']['merchant_id'];
    }
    */
  }

  $order = commerce_order_load($order_id);
  if (empty($order)) {
    $confirmation_success = FALSE;
    $confirmation_message .= ' Order not found';
  }

  if (empty($eurobank_trans_id)) {
    $confirmation_success = FALSE;
    $confirmation_message .= ' Eurobank transaction ID not set';
  }

  if ($confirmation_success) {
    watchdog('commerce_eurobank_redirect',
      'Order confirmed [Order: @order]',
      array(
        '@order' => $order_id,
      ),
      WATCHDOG_NOTICE);

    $transaction = commerce_payment_transaction_new('commerce_eurobank_redirect', $order_id);
    $transaction->instance_id = $eurobank_trans_id;
    $transaction->amount = floatval($amount) / 100;
    $transaction->currency_code = $currency;
    $transaction->payload[REQUEST_TIME] = $data;
    $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
    $transaction->remote_id = $eurobank_trans_id;
    $transaction->remote_status = 'Confirmation success';
    $transaction->message = t('Payment was successful');
    $result = commerce_payment_transaction_save($transaction);
    commerce_checkout_complete($order);

    print '[OK]';
    return;
  }
  else {
    watchdog('commerce_eurobank_redirect',
      'Confirmation error [Order: @order][Error: @error]',
      array(
        '@order' => $order_id,
        '@error' => $confirmation_message,
      ),
      WATCHDOG_WARNING);

    $transaction = commerce_payment_transaction_new('commerce_eurobank_redirect', $order_id);
    $transaction->instance_id = $eurobank_trans_id;
    $transaction->payload[REQUEST_TIME] = $data;
    $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
    $transaction->remote_id = $eurobank_trans_id;
    $transaction->remote_status = 'Confirmation error';
    $transaction->message = 'Confirmation error' . $confirmation_message;
    $result = commerce_payment_transaction_save($transaction);

    print '[NOT OK]';
    return;
  }
}

/**
 * Hook function for success.
 */
function commerce_eurobank_redirect_success() {
  commerce_static_checkout_url_redirect_with_success('commerce_eurobank_redirect');
}

/**
 * Hook function for error.
 */
function commerce_eurobank_redirect_error() {
  // Get the post data from eurobank.
  $data = $_GET;

  $trans_id = check_plain($data['ref']);
  $t = explode('_', $trans_id);
  $order_id = array_shift($t);

  $order = commerce_static_checkout_url_load_order_or_die($order_id);
  commerce_static_checkout_url_redirect_with_error('commerce_eurobank_redirect', $order, 'Credit Card error!');
}
